from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html

from django_spire.history import models


@admin.register(models.EventHistory)
class EventHistoryAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'content_object_link', 'content_type', 'created_datetime', 'event_verbose'
    )
    list_filter = ('event', 'created_datetime')
    search_fields = ('id', 'content_type__model',)
    ordering = ('-created_datetime',)

    def content_object_link(self, event_history: models.EventHistory) -> str:
        url = reverse(
            f'admin:{event_history.content_type.app_label}_{event_history.content_type.model}_change',
            args=[event_history.object_id]
        )

        return format_html(f'<a href="{url}">{event_history.content_object}</a>')

    content_object_link.short_description = 'Content Object'



from django.contrib import admin
from django.urls.base import reverse
from django.utils.html import format_html

from django_spire.history.activity.models import ActivityLog


@admin.register(ActivityLog)
class ActivityLogAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'content_object_link', 'content_type', 'created_datetime',
        'verb', 'user_link', 'recipient_link', 'information_snippet'
    )
    list_filter = ('verb', 'created_datetime')
    search_fields = ('id', 'user__first_name', 'user__last_name', 'recipient__first_name', 'recipient__last_name', 'content_type__model', 'verb')
    ordering = ('-created_datetime',)

    def content_object_link(self, activity: ActivityLog) -> str:
        url = reverse(
            f'admin:{activity.content_type.app_label}_{activity.content_type.model}_change',
            args=[activity.object_id]
        )

        return format_html(f'<a href="{url}">{activity.content_object}</a>')

    content_object_link.short_description = 'Content Object'

    def user_link(self, activity: ActivityLog) -> str:
        url = reverse('admin:auth_user_change', args=[activity.user.id])
        return format_html(f'<a href="{url}">{activity.user.get_full_name()}</a>')

    user_link.short_description = 'User'

    def recipient_link(self, activity: ActivityLog) -> str:
        if activity.recipient:
            url = reverse('admin:auth_user_change', args=[activity.recipient.id])
            return format_html(f'<a href="{url}">{activity.recipient.get_full_name()}</a>')

        return 'No Recipient'

    recipient_link.short_description = 'Recipient'

    def information_snippet(self, activity: ActivityLog) -> str:
        return (
            activity.information[:20] + '...'
            if activity.information and len(activity.information) > 20
            else activity.information or 'No Information'
        )

    information_snippet.short_description = 'Information Snippet'



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django.contrib import messages

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


def show_form_errors(request: WSGIRequest, *forms) -> None:
    for form in forms:
        for field_name, error_list in form.errors.items():
            for error in error_list.data:
                error_message = f'{field_name.title()}: {" ".join(error.messages)}'
                messages.error(request, error_message)



from django_spire.seeding.field.base import BaseFieldSeeder
from django_spire.seeding.field.enums import FieldSeederTypesEnum


class CallableFieldSeeder(BaseFieldSeeder):
    keyword = FieldSeederTypesEnum.CALLABLE

    def seed(self, manager = None, count = 1) -> list[dict]:
        return [
            {field_name: func[1]() for field_name, func in self.seeder_fields.items()}
            for _ in range(count)
        ]



from __future__ import annotations

import uuid

from django import template
from django.template.loader import render_to_string

register = template.Library()


@register.simple_tag()
def help_button(help_template, help_title: str | None = None):
    help_id = f'help-{uuid.uuid4()}'

    rendered_help_button = render_to_string(
        'spire/help/help_button.html',
        {
            'help_id': help_id,
            'help_title': help_title,
        }
    )

    rendered_help_template = render_to_string(
        'spire/help/help_modal.html',
        {
            'help_id': help_id,
            'help_title': help_title,
            'help_content': render_to_string(help_template),
        }
    )

    return rendered_help_button + rendered_help_template



from django.utils import timezone
from faker import Faker

from django_spire.seeding.field.base import BaseFieldSeeder
from django_spire.seeding.field.enums import FieldSeederTypesEnum


class CustomFieldSeeder(BaseFieldSeeder):
    keyword = FieldSeederTypesEnum.CUSTOM

    def in_order(self, values: list, index: int) -> any:
        if index >= len(values):
            raise IndexError("Index exceeds the list length in 'in_order'")
        return values[index]

    def date_time_between(self, start_date: str, end_date: str):
        faker = Faker()
        naive_dt = faker.date_time_between(start_date=start_date, end_date=end_date)
        aware_dt = timezone.make_aware(naive_dt)
        return aware_dt

    def seed(self, manager, count) -> list[dict]:
        data = []
        for i in range(count):
            row = {}
            for field_name, config in self.seeder_fields.items():
                method_name = config[1] if len(config) > 1 else field_name
                kwargs = config[2] if len(config) > 2 else {}
                method = getattr(self, method_name, None)

                if not callable(method):
                    raise ValueError(f"Custom method '{method_name}' not found for field '{field_name}'")

                if method_name == "in_order":
                    kwargs["index"] = i

                row[field_name] = method(**kwargs)

            data.append(row)
        return data



from pathlib import Path

from dandy.llm import Prompt


_RELATIVE_BASE_DIR = Path(Path(__file__).parent.parent.parent.parent.parent.resolve())


def generate_django_model_seeder_system_prompt() -> Prompt:
    return (
        Prompt()
        .title('Goal')
        .list(
            [
                'Write a DjangoModelSeeder Class for each model in the given module by the user.'
            ]
        )
        .text()
        .divider()
        .title('Return Format')
        .text('A python file that contains a DjangoModelSeeder class.')
        .text('Here is the documentation on seeding a django model.')
        .file(
            file_path=Path(_RELATIVE_BASE_DIR, 'docs/app_guides/seeding/overview.md')
        )
        .file(
            file_path=Path(_RELATIVE_BASE_DIR, 'docs/app_guides/seeding/getting_started.md')
        )
        .file(
            file_path=Path(_RELATIVE_BASE_DIR, 'docs/app_guides/seeding/faker.md')
        )
        .title('Requirements')
        .list(
            [
                'The file name should be the name of the model with a _seeder suffix.',
                'The source must be a valid Python file.',
                'Do not include triple quotes from the example.',
                'Always exclude the id field.',
            ],
        )
        .title('Specific Field Requirements.')
        .text('Follow the rules below for specific model fields.')
        .list(
            [
                "CharField with Choice Options: use faker with no method call.",
                'Foreign key: set to exclude.',
            ]
        )
        .divider()
        .title('Source Code')
        .text('Below is the source code for our seeder tools.')
        .module_source('django_spire.seeding.model.base')
        .module_source('django_spire.seeding.model.django.seeder')
        .line_break()
        .title('Example')
        .text('Below is an example of what the output should look like.')
        .module_source('django_spire.seeding.management.example')
    )


def generate_django_model_seeder_user_prompt(
        model_import: str,
        model_description: str
) -> Prompt:
    return (
        Prompt()
        .title('Django Model File')
        .module_source(model_import)
        .line_break()
        .title('Django Model Description')
        .text(model_description)
    )



from django.urls import path

from django_spire.permission import views


app_name = 'permission'

# Group
urlpatterns = [
    path('group/<int:pk>/form',
         views.group_form_view,
         name='group_form'),

    path('group/<int:pk>/delete/form',
         views.group_delete_form_view,
         name='group_delete_form'),

    path('group/<int:pk>/detail/',
         views.group_detail_view,
         name='group_detail'),

    path('group/list/',
         views.group_list_view,
         name='group_list'),

    path('group/<int:pk>/permission/<str:app_name>/ajax/',
         views.group_permission_form_ajax,
         name='group_permission_ajax'),

    path('group/<int:pk>/app/<str:app_name>/special/role/ajax/',
         views.group_special_role_form_ajax,
         name='group_special_role_ajax'),
]

# Group User
urlpatterns += [
    path('group/<int:pk>/user/form',
         views.group_user_form_view,
         name='group_user_form'),

    path('group/<int:group_pk>/user/<int:pk>/form',
         views.group_remove_user_form_view,
         name='group_user_remove_form'),
]



from django.apps import AppConfig


class NotificationAppConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'django_spire.notification.app'
    label = 'spire_notification_app'



from django.apps import AppConfig


class HistoryViewedConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    label = 'spire_history_viewed'
    name = 'django_spire.history.viewed'



import unittest
from django_spire.seeding.field.static import StaticFieldSeeder


class TestStaticFieldSeeder(unittest.TestCase):

    def test_seeds_static_fields(self):
        fields = {
            "category": ("static", "books"),
            "in_stock": ("static", True)
        }
        seeder = StaticFieldSeeder(fields=fields)
        result = seeder.seed(count=3)

        self.assertEqual(len(result), 3)
        for row in result:
            self.assertEqual(row["category"], "books")
            self.assertEqual(row["in_stock"], True)



import json
import traceback
import uuid

from dandy.recorder import Recorder
from django.contrib.auth.models import User

from django_spire.ai.models import AiInteraction


def log_ai_interaction_from_recorder(
        user: User | None = None,
        actor: str | None = None,
):
    if user is None and actor is None:
        raise ValueError('user or actor must be provided')

    def decorator(func):
        def wrapper(*args, **kwargs):
            recording_uuid = f'Recording-{uuid.uuid4()}'

            ai_interaction = AiInteraction(
                user=user,
                actor=actor,
                module_name=func.__module__,
                callable_name=func.__qualname__,
            )

            try:
                Recorder.start_recording(recording_uuid)
                return func(*args, **kwargs)

            except Exception as e:
                ai_interaction.was_successful = False
                ai_interaction.exception = str(e)

                stack_trace = '\n'.join([
                    ''.join(traceback.format_exception(None, e, e.__traceback__)).strip()
                ])

                ai_interaction.stack_trace = stack_trace

                raise e

            finally:
                Recorder.stop_recording(recording_uuid)

                ai_interaction.interaction = json.loads(Recorder.to_json_str(recording_uuid))

                ai_interaction.save()

        return wrapper

    return decorator



from __future__ import annotations

import json

from django.test import RequestFactory

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.core.shortcuts import (
    get_object_or_none,
    get_object_or_null_obj,
    model_object_from_app_label,
    process_request_body
)
from testing.dummy.models import DummyModel


class ShortcutsTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.dummy = DummyModel.objects.create(name='test')
        self.request_factory = RequestFactory()

    def test_get_object_or_null_obj_queryset_found(self) -> None:
        obj = get_object_or_null_obj(DummyModel.objects.all(), name='test')
        self.assertEqual(obj, self.dummy)

    def test_get_object_or_null_obj_queryset_not_found(self) -> None:
        obj = get_object_or_null_obj(DummyModel.objects.all(), name='nonexistent')
        self.assertIsNone(obj.pk)
        self.assertEqual(obj.name, '')

    def test_get_object_or_null_obj_model_found(self) -> None:
        obj = get_object_or_null_obj(DummyModel, name='test')
        self.assertEqual(obj, self.dummy)

    def test_get_object_or_null_obj_model_not_found(self) -> None:
        obj = get_object_or_null_obj(DummyModel, name='nonexistent')
        self.assertIsNone(obj.pk)
        self.assertEqual(obj.name, '')

    def test_get_object_or_none_found(self) -> None:
        obj = get_object_or_none(DummyModel, pk=self.dummy.pk)
        self.assertEqual(obj, self.dummy)

    def test_get_object_or_none_not_found(self) -> None:
        obj = get_object_or_none(DummyModel, pk=9999)
        self.assertIsNone(obj)

    def test_process_request_body(self) -> None:
        data = {'data': {'key': 'value'}}

        request = self.request_factory.post(
            '/dummy-url/',
            data=json.dumps(data),
            content_type='application/json'
        )

        result = process_request_body(request)
        self.assertEqual(result, data['data'])

    def test_model_object_from_app_label_found(self) -> None:
        obj = model_object_from_app_label('dummy', 'dummymodel', self.dummy.pk)
        self.assertEqual(obj, self.dummy)

    def test_model_object_from_app_label_not_found(self) -> None:
        obj = model_object_from_app_label('dummy', 'dummymodel', 9999)
        self.assertIsNone(obj)

    def test_model_object_from_app_label_contenttype_not_found(self) -> None:
        obj = model_object_from_app_label('nonexistent', 'nonexistent', 1)
        self.assertIsNone(obj)



from __future__ import annotations

import json

from django.template import loader
from django.utils.safestring import mark_safe
from django.forms import Widget


class MultipleWidget(Widget):
    needs_multipart_form = True
    template_name = 'spire/file/widget/multiple_file_widget.html'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def value_from_datadict(self, data, files, name):
        return json.loads(data.get(f'{name}_data'))

    def render(self, name, value, attrs=None, renderer=None):
        context = self.get_context(name, value, attrs)
        template = loader.get_template(self.template_name).render(context)
        return mark_safe(template)


class SingleFileWidget(Widget):
    needs_multipart_form = True
    template_name = 'spire/file/widget/single_file_widget.html'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def value_from_datadict(self, data, files, name) -> dict:
        return json.loads(data.get(f'{name}_data'))

    def render(self, name, value, attrs=None, renderer=None):
        context = self.get_context(name, value, attrs)
        template = loader.get_template(self.template_name).render(context)
        return mark_safe(template)



from __future__ import annotations

from django.contrib.auth import logout
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect
from django.urls import reverse


@login_required()
def login_redirect_view(request, **kwargs):
    if not request.user.is_authenticated:
        return HttpResponseRedirect(reverse('user_account:authentication:admin:login'))
    else:
        # MFA implementation if required
        # profile = request.user.profile
        #
        # if profile.requires_mfa():
        #     return HttpResponseRedirect(reverse('user_account:authentication:mfa:redirect:notification'))

        return HttpResponseRedirect(reverse('home:home'))


@login_required()
def logout_redirect_view(request, **kwargs):
    logout(request)
    return HttpResponseRedirect(reverse('user_account:authentication:admin:login'))



from __future__ import annotations

from django.contrib.contenttypes.fields import GenericRelation
from django.db import models
from django.utils.timezone import localtime
from typing_extensions import TYPE_CHECKING

from django_spire.history.choices import EventHistoryChoices
from django_spire.history.models import EventHistory

if TYPE_CHECKING:
    from django.contrib.auth.models import User


class HistoryModelMixin(models.Model):
    is_active = models.BooleanField(default=True, editable=False)
    is_deleted = models.BooleanField(default=False, editable=False)

    event_history = GenericRelation(
        EventHistory,
        related_query_name='spire_event_history',
        editable=False
    )

    created_datetime = models.DateTimeField(default=localtime, editable=False)

    def save(self, *args, **kwargs) -> None:
        super().save(*args, **kwargs)

        if self.pk:
            self.event_history.create(event=EventHistoryChoices.UPDATED)
        else:
            self.event_history.create(event=EventHistoryChoices.CREATED)

    def add_view(self, user: User) -> None:
        self.views.create(user=user)

    def is_viewed(self, user: User) -> None:
        return self.views.filter(user=user).exists()

    def set_active(self) -> None:
        self.is_active = True
        self.save()
        self.event_history.create(event=EventHistoryChoices.ACTIVE)

    def set_deleted(self) -> None:
        self.is_deleted = True
        self.save()
        self.event_history.create(event=EventHistoryChoices.DELETED)

    def set_inactive(self) -> None:
        self.is_active = False
        self.save()
        self.event_history.create(event=EventHistoryChoices.INACTIVE)

    def un_set_deleted(self) -> None:
        self.is_deleted = False
        self.save()
        self.event_history.create(event=EventHistoryChoices.UNDELETED)

    class Meta:
        abstract = True



from __future__ import annotations

from dandy.llm import Prompt
from dandy.intel import BaseIntel

from django_spire.core.converters import django_to_pydantic_model, fake_model_field_value
from django_spire.seeding.field.base import BaseFieldSeeder
from django_spire.seeding.field.enums import FieldSeederTypesEnum
from django_spire.seeding.intelligence.bots.field_seeding_bots import LlmFieldSeedingBot


class DjangoFieldLlmSeeder(BaseFieldSeeder):
    keyword = FieldSeederTypesEnum.LLM

    def seed(self, model_seeder, count: int = 1,) -> list[dict]:

        include_fields = list(self.seeder_fields.keys())

        seed_intel_class = django_to_pydantic_model(
            model_class=model_seeder.model_class,
            base_class=BaseIntel,
            include_fields=include_fields
        )

        class SeedingIntel(BaseIntel):
            items: list[seed_intel_class]

            def __iter__(self):
                return iter(self.items)

        prompt = (
            Prompt()
            .heading('Seed Count')
            .text(f'Create {count} {model_seeder.model_class.__name__}')
            .heading('General Seeding Rules')
            .list([
                'Create data for each field provided.'
            ])
            .heading('Field Rules & Context')
            .prompt(self.field_prompt)
            .text(f'Create {count} {model_seeder.model_class.__name__}')
        )

        intel_data = LlmFieldSeedingBot.process(
            prompt=prompt,
            intel_class=SeedingIntel
        )

        return intel_data

    @property
    def field_prompt(self) -> Prompt:
        if any(len(info) > 1 for info in self.seeder_fields.values()):
            field_prompt = (
                Prompt()
                .heading('Fields Context Data')
                .list([
                    f'{name}: {info[1]}'
                    for name, info in self.seeder_fields.items()
                    if info[0] == 'llm' and len(info) > 1
                ])
            )
        else:
            field_prompt = Prompt()

        return field_prompt



class DjangoFieldFakerSeeder(BaseFieldSeeder):
    keyword = FieldSeederTypesEnum.FAKER

    def seed(self, model_seeder, count = 1) -> list[dict]:
        data = []
        for i in range(count):
            row = {}
            for field_name, faker_config in self.seeder_fields.items():
                faker_method = faker_config[1:] if len(faker_config) > 1 else None
                try:
                    row[field_name] = fake_model_field_value(
                        model_class=model_seeder.model_class,
                        field_name=field_name,
                        faker_method=faker_method
                    )
                except Exception as e:
                    raise ValueError(f"Error generating faker value for field '{field_name}': {e}")
            data.append(row)
        return data



from __future__ import annotations

from django.apps import AppConfig


class HistoryConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    label = 'spire_history'
    name = 'django_spire.history'



from django.contrib.auth.models import Group, User
from django.db import models
from django.utils.formats import localize
from django.utils.timezone import now


class AiInteraction(models.Model):
    user = models.ForeignKey(User, blank=True, null=True, on_delete=models.SET_NULL)
    actor = models.CharField(max_length=100)
    user_email = models.EmailField(blank=True, null=True)
    user_first_name = models.CharField(max_length=100, blank=True, null=True)
    user_last_name = models.CharField(max_length=100, blank=True, null=True)

    module_name = models.TextField()
    callable_name = models.TextField()

    was_successful = models.BooleanField(default=True)
    exception = models.TextField(blank=True, null=True)
    stack_trace = models.TextField(blank=True, null=True)

    created_datetime = models.DateTimeField(default=now)

    interaction = models.JSONField(blank=True, null=True)

    def __str__(self):
        return f'"{self.actor}" ai interaction with "{self.module_name}.{self.callable_name}" on "{localize(self.created_datetime)}"'

    def save(self, *args, **kwargs):
        if self.user:
            if self.actor is None:
                self.actor = self.user.get_full_name()
            if self.user_email is None:
                self.user_email = self.user.email
            if self.user_first_name is None:
                self.user_first_name = self.user.first_name
            if self.user_last_name is None:
                self.user_last_name = self.user.last_name

        super().save(*args, **kwargs)

    class Meta:
        db_table = 'spire_ai_interaction'




from django.urls import path

from django_spire.authentication.views import redirect_views


app_name = 'authentication'

urlpatterns = [
    path('login/redirect/',
         redirect_views.login_redirect_view,
         name='login'),

    path('logout/redirect/',
         redirect_views.logout_redirect_view,
         name='logout'),
]



from django.urls import include, path


app_name = '__init__'

urlpatterns  = [
    path('', include('django_spire.authentication.mfa.urls.page_urls', namespace='page')),
    path('', include('django_spire.authentication.mfa.urls.redirect_urls', namespace='redirect')),
]



# Generated by Django 5.2 on 2025-04-11 21:12

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('spire_notification', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailNotification',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subject', models.CharField(max_length=128)),
                ('email', models.EmailField(max_length=254)),
                ('notification', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='spire_notification.notification')),
            ],
            options={
                'verbose_name': 'Email Notification',
                'verbose_name_plural': 'Email Notifications',
                'db_table': 'spire_notification_email',
            },
        ),
    ]



from django.urls import path

from django_spire.user_account import views

app_name = 'user_account'

urlpatterns = [
    path('register/user/',
         views.register_user_form_view,
         name='register_user_form'),

    path('user/<int:pk>/detail/',
         views.user_detail_page_view,
         name='detail'),

    path('user/list/',
         views.user_list_page_view,
         name='list'),

    path('user/<int:pk>/form',
         views.user_form_page_view,
         name='form'),

    path('user/<int:pk>/toggle/form/',
         views.user_status_form_page_view,
         name='toggle_user_status'),
]



from django.urls import path, include


app_name = 'authentication'

urlpatterns = [
    path('', include('django_spire.authentication.urls.admin_urls', namespace='admin')),
    path('redirect/', include('django_spire.authentication.urls.redirect_urls', namespace='redirect')),
    path('mfa/', include('django_spire.authentication.mfa.urls', namespace='mfa')),
]



from __future__ import annotations

import json

from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.urls import reverse

import django_glue as dg
from django_glue.utils import serialize_to_json

from django_spire.core.redirect import safe_redirect_url
from django_spire.breadcrumb.models import Breadcrumbs
from django_spire.form.confirmation_forms import (
    DeleteConfirmationForm,
    ConfirmationForm
)
from django_spire.form.utils import show_form_errors
from django_spire.history.utils import add_form_activity
from django_spire.pagination.pagination import paginate_list
from django_spire.permission.decorators import permission_required
from django_spire.permission.models import PortalUser
from django_spire.permission.permissions import (
    generate_group_perm_data,
    generate_user_perm_data
)
from django_spire.user_account import forms
from django_spire.views import portal_views


def register_user_form_view(request):
    # Is this the best way to initialize a null object in glue?
    dg.glue_model_object(request, 'portal_user', PortalUser(), 'view')

    if request.method == 'POST':
        user_form = forms.RegisterUserForm(request.POST)

        # Checks to see if all forms are valid.
        if user_form.is_valid():
            user = user_form.save()

            # Add form activity. This needs to be improved.
            add_form_activity(user, 0, request.user)

            return HttpResponseRedirect(reverse('user_account:list'))

        show_form_errors(request, user_form)
    else:
        # If the form has an initial it will override the defaults.
        user_form = forms.RegisterUserForm()

    context_data = {
        # Todo: Function that takes in all of the forms and dumps the data here?
        'user_form_data': json.dumps(user_form.data, cls=DjangoJSONEncoder),
    }

    crumbs = Breadcrumbs()
    crumbs.add_breadcrumb(name='Users', href=reverse('user_account:list'))
    crumbs.add_breadcrumb(name='Register New User')

    return portal_views.template_view(
        request,
        context_data=context_data,
        page_title='Register',
        page_description='New User',
        breadcrumbs=crumbs,
        template='spire/user_account/page/register_user_form_page.html'
    )


@permission_required('permission.view_portaluser')
def user_detail_page_view(request, pk):
    user = get_object_or_404(PortalUser, pk=pk)
    group_list = user.groups.all()

    context_data = {
        'user': user,
        'group_list': group_list,
        'group_list_permission_data': [generate_group_perm_data(group) for group in group_list],
        'user_perm_data': generate_user_perm_data(user)
    }

    return portal_views.detail_view(
        request,
        context_data=context_data,
        obj=user,
        template='spire/user_account/page/user_detail_page.html'
    )


@permission_required('permission.view_portaluser')
def user_list_page_view(request):
    user_list = PortalUser.objects.all()

    context_data = {
        'user_list': paginate_list(user_list, page_number=request.GET.get('page', 1))
    }

    return portal_views.list_view(
        request,
        context_data=context_data,
        model=PortalUser,
        template='spire/user_account/page/user_list_page.html'
    )


@permission_required('permission.change_portaluser')
def user_form_page_view(request, pk):
    portal_user = get_object_or_404(PortalUser, pk=pk)
    dg.glue_model_object(request, 'portal_user', portal_user, 'view')

    if request.method == 'POST':
        form = forms.UserForm(request.POST, instance=portal_user)

        if form.is_valid():
            portal_user = form.save()
            add_form_activity(portal_user, pk, request.user)

            return HttpResponseRedirect(reverse('user_account:list'))
    else:
        form = forms.UserForm(instance=portal_user)

    context_data = {
        'portal_user': portal_user,
        'initial_data': serialize_to_json(form.data)
    }

    return portal_views.model_form_view(
        request,
        form=form,
        obj=portal_user,
        context_data=context_data,
        template='spire/user_account/page/user_form_page.html'
    )


@permission_required('permission.delete_portaluser')
def user_status_form_page_view(request, pk):
    user = get_object_or_404(PortalUser, pk=pk)

    form_class = DeleteConfirmationForm if user.is_active else ConfirmationForm

    if request.method == 'POST':
        form = form_class(request.POST)

        if form.is_valid():
            user.is_active = not user.is_active
            user.save()

            toggle_verb = 'activated' if user.is_active else 'deactivated'

            user.add_activity(
                user=request.user,
                verb=toggle_verb,
                information=f'{request.user.get_full_name()} {toggle_verb} user "{user.get_full_name()}".'
            )

            return_url = safe_redirect_url(request)
            return HttpResponseRedirect(return_url)

    form = form_class()
    toggle_verb = 'Activate' if user.is_active else 'Deactivate'

    def update_breadcrumbs(breadcrumbs: Breadcrumbs):
        breadcrumbs.add_breadcrumb(name=toggle_verb)

    return portal_views.form_view(
        request,
        form=form,
        verb=toggle_verb,
        obj=user,
        breadcrumbs_func=update_breadcrumbs
    )







class FieldOverride:
    def __init__(self, seeder_cls):
        self.seeder_cls = seeder_cls
        self.overrides = {}

    def filter(self, **kwargs):
        self.overrides.update(kwargs)
        return self

    def seed(self, count=1):
        return self.seeder_cls.seed(count=count, fields=self.overrides)

    def seed_database(self, count=1):
        return self.seeder_cls.seed_database(count=count, fields=self.overrides)



from __future__ import annotations

from django.contrib.contenttypes.fields import GenericRelation

from django.db import models


class FileModelMixin(models.Model):
    files = GenericRelation('file.File', editable=False)

    class Meta:
        abstract = True



import datetime
from decimal import Decimal

from uuid import UUID

from django.db import models


MODEL_FIELD_TYPE_TO_TYPE_MAP = {
    models.AutoField.__name__: int,
    models.BigAutoField.__name__: int,
    models.SmallAutoField.__name__: int,
    models.BooleanField.__name__: bool,
    models.CharField.__name__: str,
    models.CommaSeparatedIntegerField.__name__: int,
    models.DateField.__name__: datetime.date,
    models.DateTimeField.__name__: datetime.datetime,
    models.DecimalField.__name__: Decimal,
    models.DurationField.__name__: str,
    models.EmailField.__name__: str,
    models.FileField.__name__: str,
    models.FilePathField.__name__: str,
    models.FloatField.__name__: float,
    models.ForeignKey.__name__: int | UUID,
    models.OneToOneField.__name__: int | UUID,
    models.ManyToManyField.__name__: list[int],
    models.IntegerField.__name__: int,
    models.BigIntegerField.__name__: int,
    models.SmallIntegerField.__name__: int,
    models.IPAddressField.__name__: str,
    models.GenericIPAddressField.__name__: str,
    models.JSONField.__name__: dict | list,
    models.NullBooleanField.__name__: bool | None,
    models.PositiveBigIntegerField.__name__: int,
    models.PositiveIntegerField.__name__: int,
    models.PositiveSmallIntegerField.__name__: int,
    models.SlugField.__name__: str | UUID,
    models.TextField.__name__: str,
    models.TimeField.__name__: datetime.time,
    models.URLField.__name__: str,
    models.UUIDField.__name__: UUID,
}



from __future__ import annotations

from django_spire.notification.models import Notification


def process_notifications() -> str:
    for notification in Notification.objects.filter(is_sent=False):
        notification.send()

    return 'Successfully Completed'



from django_spire.seeding.intelligence.prompts.generic_relationship_selection_prompt import generic_relationship_selection_prompt
from django_spire.seeding.intelligence.prompts.foreign_key_selection_prompt import foreign_key_selection_prompt
from django_spire.seeding.intelligence.prompts.objective_prompt import objective_prompt
from django_spire.seeding.intelligence.prompts.negation_prompt import negation_prompt


from django.urls import path

from module import views


app_name = 'spirechildapp'

urlpatterns = [
    path('<int:pk>/delete/form/', views.spireparentapp_spirechildapp_delete_view, name='delete'),
    path('<int:pk>/delete/form/modal/', views.spireparentapp_spirechildapp_delete_form_modal_view, name='delete_form_modal'),
    path('<int:pk>/detail/', views.spireparentapp_spirechildapp_detail_view, name='detail'),
    path('<int:pk>/form/content/', views.spireparentapp_spirechildapp_form_content_modal_view, name='form_modal_content'),
    path('<int:pk>/form/', views.spireparentapp_spirechildapp_form_view, name='form'),
    path('list/', views.spireparentapp_spirechildapp_list_view, name='list'),
]



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django.http import HttpResponseRedirect, JsonResponse
from django.shortcuts import get_object_or_404
from django.urls import reverse
from django.views.decorators.http import require_POST

from django_spire.core.shortcuts import (
    get_object_or_null_obj,
    process_request_body
)
from django_spire.form.confirmation_forms import DeleteConfirmationForm
from django_spire.history.utils import add_form_activity
from django_spire.permission import forms, models
from django_spire.permission.constants import PERMISSION_MODELS_DICT
from django_spire.permission.decorators import permission_required
from django_spire.permission.models import PortalUser
from django_spire.permission.permissions import (
    generate_group_perm_data,
    GroupPermissions
)
from django_spire.permission.utils import (
    add_users_to_group,
    perm_level_to_int,
    perm_level_to_django_permission
)
from django_spire.views import portal_views

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from django.template.response import TemplateResponse


@permission_required('permission.view_portalgroup')
def group_detail_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    group = get_object_or_404(models.PortalGroup, pk=pk)

    context_data = {
        'group': group,
        'permission_data': generate_group_perm_data(group, with_special_role=True),
        'user_list': group.user_set.all()
    }

    return portal_views.detail_view(
        request,
        context_data=context_data,
        obj=group,
        template='spire/permission/page/group_detail_page.html'
    )


@permission_required('permission.view_portalgroup')
def group_list_view(request: WSGIRequest) -> TemplateResponse:
    group_list = models.PortalGroup.objects.all().prefetch_related('permissions')

    context_data = {
        'group_list': group_list,
        'group_list_permission_data': [
            generate_group_perm_data(group)
            for group in group_list
        ]
    }

    return portal_views.list_view(
        request,
        context_data=context_data,
        model=models.PortalGroup,
        template='spire/permission/page/group_list_page.html'
    )


@permission_required('permission.change_portalgroup')
def group_form_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    group = get_object_or_null_obj(models.PortalGroup, pk=pk)

    if request.method == 'POST':
        form = forms.GroupForm(request.POST, instance=group)

        if form.is_valid():
            group = form.save()
            add_form_activity(group, pk, request.user)

            return_url = reverse('permission:group_list')
            return HttpResponseRedirect(return_url)

    form = forms.GroupForm(instance=group)

    context_data = {'group': group}

    return portal_views.model_form_view(
        request,
        form=form,
        context_data=context_data,
        obj=group,
    )


@permission_required('permission.add_portalgroup')
def group_user_form_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    group = get_object_or_404(models.PortalGroup, pk=pk)

    if request.method == 'POST':
        form = forms.GroupUserForm(request.POST, group=group)

        if form.is_valid():
            user_list = form.cleaned_data.get('available_users')
            add_users_to_group(group, user_list)

            group.add_activity(
                user=request.user,
                verb='added',
                information=f'{request.user.get_full_name()} added {len(user_list)} users to the group "{group.name}".'
            )

            return_url = reverse('permission:group_detail', kwargs={'pk': pk})
            return HttpResponseRedirect(return_url)

    form = forms.GroupUserForm(group=group)

    def crumbs(breadcrumbs) -> None:
        breadcrumbs.add_breadcrumb(name='Add Users')

    context_data = {'form_description': ''}

    return portal_views.form_view(
        request,
        form=form,
        obj=group,
        context_data=context_data,
        breadcrumbs_func=crumbs,
    )


@permission_required('permission.delete_portalgroup')
def group_delete_form_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    group = get_object_or_404(models.PortalGroup, pk=pk)
    return_url = reverse('permission:group_list')

    return portal_views.delete_form_view(
        request,
        obj=group,
        delete_func=group.delete,
        return_url=return_url
    )


@permission_required('permission.delete_portaluser')
def group_remove_user_form_view(
    request: WSGIRequest,
    group_pk: int,
    pk: int
) -> HttpResponseRedirect | TemplateResponse:
    group = get_object_or_404(models.PortalGroup, pk=group_pk)
    user = get_object_or_404(PortalUser, pk=pk)

    if request.method == 'POST':
        form = DeleteConfirmationForm(request.POST, obj=user)

        if form.is_valid():
            user.groups.remove(group)

            group.add_activity(
                user=request.user,
                verb='removed',
                information=f'{request.user.get_full_name()} removed {user.get_full_name()} from the group "{group.name}".'
            )

            return_url = reverse('permission:group_detail', kwargs={'pk': group_pk})
            return HttpResponseRedirect(return_url)

    form = DeleteConfirmationForm(request.GET, obj=user)

    def get_breadcrumbs(breadcrumbs) -> None:
        breadcrumbs = group.breadcrumbs()
        breadcrumbs.add_breadcrumb(name=user.get_full_name())
        breadcrumbs.add_breadcrumb(name='Remove')

    return portal_views.form_view(
        request,
        form=form,
        verb=f'remove {user.get_full_name()} from',
        obj=group,
        breadcrumbs_func=get_breadcrumbs
    )


@require_POST
@permission_required('permission.change_portalgroup')
def group_permission_form_ajax(
    request: WSGIRequest,
    pk: int,
    app_name: str
) -> JsonResponse:
    if request.method == 'POST':
        error_message = 'App Does Not Exist.'

        if app_name.lower() in PERMISSION_MODELS_DICT:
            body = process_request_body(request)
            perm_level = perm_level_to_int(body.get('perm_level'))
            error_message = 'Please provide a valid permission level'

            if isinstance(perm_level, int) and 4 >= perm_level >= 0:
                group = get_object_or_404(models.PortalGroup, pk=pk)
                group_perm_helper = GroupPermissions(group, model_key=app_name)

                django_permission_verbose = perm_level_to_django_permission(
                    perm_level=perm_level,
                    app_label=group_perm_helper.model_permissions.app_label,
                    model_name=group_perm_helper.model_permissions.model_name
                )

                error_message = 'You do not have permission to change this app.'

                if request.user.has_perm(django_permission_verbose) or perm_level == 0:
                    group_perm_helper.update_perms(perm_level)
                    return JsonResponse({'message': 'Updated Successfully!', 'status': 200})

        return JsonResponse({'message': error_message, 'status': 400})


@require_POST
@permission_required('permission.change_portalgroup')
def group_special_role_form_ajax(
    request: WSGIRequest,
    pk: int,
    app_name: str
) -> JsonResponse:
    if request.method == 'POST':
        error_message = 'App Does Not Exist.'

        if app_name.lower() in PERMISSION_MODELS_DICT:
            body = process_request_body(request)
            grant_special_role_access = body.get('grant_special_role_access')
            codename = body.get('codename')
            error_message = 'models.PortalUser does not have permission.'

            if request.user.has_perm('permission.change_portalgroup'):
                group = get_object_or_404(models.PortalGroup, pk=pk)
                group_perm_helper = GroupPermissions(group, model_key=app_name)

                if grant_special_role_access:
                    group_perm_helper.add_special_role(codename)
                else:
                    group_perm_helper.remove_special_role(codename)

                return JsonResponse({'message': 'Updated Successfully!', 'status': 200})

        return JsonResponse({'message': error_message, 'status': 400})



from __future__ import annotations

import warnings
import whisper

from enum import Enum
from pathlib import Path

from django.core.files.storage import FileSystemStorage
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_protect


warnings.simplefilter(action='ignore', category=FutureWarning)


class WhisperModel(Enum):
    TINY = 'tiny'
    BASE = 'base'
    SMALL = 'small'
    MEDIUM = 'medium'
    LARGE = 'large'
    TURBO = 'turbo'


model = whisper.load_model(WhisperModel.TINY.value)


@csrf_protect
def transcribe_audio(request):
    if request.method == 'POST':
        file = request.FILES.get('audio')

        if not file:
            return JsonResponse({'error': 'No audio file provided'}, status=400)

        storage = FileSystemStorage()
        location = storage.save(file.name, file)
        path = Path(storage.path(location))

        try:
            result = model.transcribe(str(path))
            transcription = result['text']
        except Exception as e:
            return JsonResponse({'error': str(e)}, status=500)
        finally:
            if path.exists():
                path.unlink()

        return JsonResponse({'transcription': transcription}, status=200)

    return JsonResponse({'error': 'Invalid request method'}, status=405)



# Generated by Django 5.1.7 on 2025-03-27 22:22

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='EventHistory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField(editable=False)),
                ('event', models.CharField(choices=[('crea', 'Created'), ('upda', 'Updated'), ('acti', 'Active'), ('inac', 'Inactive'), ('dele', 'Deleted'), ('unde', 'Un-Deleted')], default='upda', max_length=4)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.localtime)),
                ('content_type', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='spire_eventhistory', to='contenttypes.contenttype')),
            ],
            options={
                'verbose_name': 'Event History',
                'verbose_name_plural': 'Event History',
                'db_table': 'spire_history_event_history',
            },
        ),
    ]



import json

from django import forms
from django.contrib import admin

from django_spire.ai import models


class InteractionField(forms.Textarea):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.template_name = 'spire/ai/forms/widgets/ai_interaction_widget.html'

    def get_context(self, *args, **kwargs):
        context = super().get_context(*args, **kwargs)
        if context['widget']['value'] is None:
            context['interaction_dict'] = {}
        else:
            context['interaction_dict'] = json.loads(context['widget']['value'])

        return context

class AiInteractionModelForm(forms.ModelForm):
    interaction = forms.JSONField(widget=InteractionField)

    class Meta:
        model = models.AiInteraction
        fields = '__all__'

@admin.register(models.AiInteraction)
class AiInteractionAdmin(admin.ModelAdmin):
    form = AiInteractionModelForm

    list_display = ('actor', 'module_name', 'callable_name', 'created_datetime', 'was_successful')
    list_filter = ('module_name', 'callable_name')
    search_fields = ('actor', 'user_email', 'user_first_name', 'user_last_name', 'module_name', 'callable_name')
    ordering = ('-created_datetime',)


    readonly_fields = [
        'actor',
        'created_datetime',
        'module_name',
        'callable_name',
        'was_successful',
        'user',
        'user_email',
        'user_first_name',
        'user_last_name',
        'exception',
        'stack_trace',
    ]
    fields = [
        'actor',
        'user',
        'created_datetime',
        'module_name',
        'callable_name',
        'was_successful',
        'interaction',
        'user_email',
        'user_first_name',
        'user_last_name',
        'exception',
        'stack_trace',
    ]



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django.shortcuts import get_object_or_404

from django_spire.views import portal_views

from django_spire.notification.app.models import AppNotification

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from django.template.response import TemplateResponse

def app_notification_list_view(request: WSGIRequest) -> TemplateResponse:
    app_notifification_list = AppNotification.objects.by_user(request.user).order_by('-created_datetime')

    return portal_views.list_view(
        request,
        context_data={'notification_list': app_notifification_list},
        model=AppNotification,
        template='spire/notification/app/page/list_page.html'
    )



from django.apps import AppConfig


class HistoryActivityConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    label = 'spire_history_activity'
    name = 'django_spire.history.activity'



from django.urls import path

from django_spire.notification.app.views import ajax


app_name = 'spire_notification'

urlpatterns = [
    path('check/notification/ajax/',
         ajax.check_new_notifications_ajax_view,
         name='check_new'),

    path('set_viewed/notification/ajax/',
         ajax.set_notifications_as_viewed_ajax,
         name='set_viewed'),
]



# Generated by Django 5.2 on 2025-04-07 03:52

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AiInteraction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('actor', models.CharField(max_length=100)),
                ('user_email', models.EmailField(blank=True, max_length=254, null=True)),
                ('user_first_name', models.CharField(blank=True, max_length=100, null=True)),
                ('user_last_name', models.CharField(blank=True, max_length=100, null=True)),
                ('module_name', models.TextField()),
                ('callable_name', models.TextField()),
                ('was_successful', models.BooleanField(default=True)),
                ('exception', models.TextField(blank=True, null=True)),
                ('stack_trace', models.TextField(blank=True, null=True)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.now)),
                ('interaction', models.JSONField(blank=True, null=True)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'spire_ai_interaction',
            }
        ),
    ]
